# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara_plugin_vanilla.plugins.vanilla import confighints_helper
from sahara_plugin_vanilla.tests.unit import base as sahara_base


class ConfigHintsHelperTest(sahara_base.SaharaTestCase):
    @mock.patch('sahara.plugins.utils.load_hadoop_xml_defaults',
                return_value=[])
    def test_get_possible_hive_config_from(self, load_hadoop_xml_defaults):
        expected_config = {
            'configs': [],
            'params': {}
        }
        actual_config = confighints_helper.get_possible_hive_config_from(
            'sample-config.xml')
        load_hadoop_xml_defaults.assert_called_once_with(
            'sample-config.xml', 'sahara_plugin_vanilla')
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara.plugins.utils.load_hadoop_xml_defaults',
                return_value=[])
    @mock.patch('sahara.plugins.edp.get_possible_mapreduce_configs',
                return_value=[])
    def test_get_possible_mapreduce_config_from(
            self, get_possible_mapreduce_configs, load_hadoop_xml_defaults):
        expected_config = {
            'configs': [],
        }
        actual_config = confighints_helper.get_possible_mapreduce_config_from(
            'sample-config.xml')
        load_hadoop_xml_defaults.assert_any_call('sample-config.xml',
                                                 'sahara_plugin_vanilla')
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara.plugins.utils.load_hadoop_xml_defaults',
                return_value=[])
    def test_get_possible_pig_config_from(
            self, load_hadoop_xml_defaults):
        expected_config = {
            'configs': [],
            'args': [],
            'params': {}
        }
        actual_config = confighints_helper.get_possible_pig_config_from(
            'sample-config.xml')
        load_hadoop_xml_defaults.assert_called_once_with(
            'sample-config.xml', 'sahara_plugin_vanilla')
        self.assertEqual(expected_config, actual_config)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config as c
from sahara_plugin_vanilla.tests.unit import base


class VanillaTwoConfigTestCase(base.SaharaTestCase):
    def test_get_hadoop_dirs(self):
        ng = FakeNG(storage_paths=['/vol1', '/vol2'])
        dirs = c._get_hadoop_dirs(ng)
        expected = {
            'hadoop_name_dirs': ['/vol1/hdfs/namenode',
                                 '/vol2/hdfs/namenode'],
            'hadoop_data_dirs': ['/vol1/hdfs/datanode',
                                 '/vol2/hdfs/datanode'],
            'hadoop_log_dir': '/vol1/hadoop/logs',
            'hadoop_secure_dn_log_dir': '/vol1/hadoop/logs/secure',
            'yarn_log_dir': '/vol1/yarn/logs'
        }
        self.assertEqual(expected, dirs)


class FakeNG(object):
    def __init__(self, storage_paths=None):
        self.paths = storage_paths

    def storage_paths(self):
        return self.paths



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from sahara.plugins import provisioning as p
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla import versionfactory as vhf


class VanillaProvider(p.ProvisioningPluginBase):
    def __init__(self):
        self.version_factory = vhf.VersionFactory.get_instance()

    def get_description(self):
        return _('The Apache Vanilla plugin provides the ability to launch '
                 'upstream Vanilla Apache Hadoop cluster without any '
                 'management consoles. It can also deploy the Oozie '
                 'component.')

    def _get_version_handler(self, hadoop_version):
        return self.version_factory.get_version_handler(hadoop_version)

    def get_node_processes(self, hadoop_version):
        return self._get_version_handler(hadoop_version).get_node_processes()

    def get_labels(self):
        default = {'enabled': {'status': True}, 'stable': {'status': True}}
        result = {'plugin_labels': copy.deepcopy(default)}
        result['version_labels'] = {
            version: copy.deepcopy(default) for version in self.get_versions()
        }
        return result

    def get_versions(self):
        return self.version_factory.get_versions()

    def get_title(self):
        return "Vanilla Apache Hadoop"

    def get_configs(self, hadoop_version):
        return self._get_version_handler(hadoop_version).get_plugin_configs()

    def configure_cluster(self, cluster):
        return self._get_version_handler(
            cluster.hadoop_version).configure_cluster(cluster)

    def start_cluster(self, cluster):
        return self._get_version_handler(
            cluster.hadoop_version).start_cluster(cluster)

    def validate(self, cluster):
        return self._get_version_handler(
            cluster.hadoop_version).validate(cluster)

    def scale_cluster(self, cluster, instances):
        return self._get_version_handler(
            cluster.hadoop_version).scale_cluster(cluster, instances)

    def decommission_nodes(self, cluster, instances):
        return self._get_version_handler(
            cluster.hadoop_version).decommission_nodes(cluster, instances)

    def validate_scaling(self, cluster, existing, additional):
        return self._get_version_handler(
            cluster.hadoop_version).validate_scaling(cluster, existing,
                                                     additional)

    def get_edp_engine(self, cluster, job_type):
        return self._get_version_handler(
            cluster.hadoop_version).get_edp_engine(cluster, job_type)

    def get_edp_job_types(self, versions=None):
        res = {}
        for vers in self.version_factory.get_versions():
            if not versions or vers in versions:
                vh = self.version_factory.get_version_handler(vers)
                res[vers] = vh.get_edp_job_types()
        return res

    def get_edp_config_hints(self, job_type, version):
        version_handler = (
            self.version_factory.get_version_handler(version))
        return version_handler.get_edp_config_hints(job_type)

    def get_open_ports(self, node_group):
        return self._get_version_handler(
            node_group.cluster.hadoop_version).get_open_ports(node_group)

    def on_terminate_cluster(self, cluster):
        return self._get_version_handler(
            cluster.hadoop_version).on_terminate_cluster(cluster)

    def recommend_configs(self, cluster, scaling=False):
        return self._get_version_handler(
            cluster.hadoop_version).recommend_configs(cluster, scaling)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re

from oslo_log import log as logging

from sahara.plugins import castellan_utils as castellan
from sahara.plugins import conductor
from sahara.plugins import context
from sahara_plugin_vanilla.plugins.vanilla import utils as u


LOG = logging.getLogger(__name__)


def get_datanodes_status(cluster):
    statuses = {}
    namenode = u.get_namenode(cluster)
    status_regexp = r'^Hostname: (.*)\nDecommission Status : (.*)$'
    matcher = re.compile(status_regexp, re.MULTILINE)
    dfs_report = namenode.remote().execute_command(
        'sudo su - -c "hdfs dfsadmin -report" hadoop')[1]

    for host, status in matcher.findall(dfs_report):
        statuses[host] = status.lower()

    return statuses


def get_nodemanagers_status(cluster):
    statuses = {}
    resourcemanager = u.get_resourcemanager(cluster)
    status_regexp = r'^(\S+):\d+\s+(\w+)'
    matcher = re.compile(status_regexp, re.MULTILINE)
    yarn_report = resourcemanager.remote().execute_command(
        'sudo su - -c "yarn node -all -list" hadoop')[1]

    for host, status in matcher.findall(yarn_report):
        statuses[host] = status.lower()

    return statuses


def get_oozie_password(cluster):
    cluster = conductor.cluster_get(context.ctx(), cluster)
    extra = cluster.extra.to_dict()
    if 'oozie_pass_id' not in extra:
        extra['oozie_pass_id'] = u.generate_random_password()
        conductor.cluster_update(context.ctx(), cluster, {'extra': extra})
    return castellan.get_secret(extra['oozie_pass_id'])


def delete_oozie_password(cluster):
    extra = cluster.extra.to_dict()
    if 'oozie_pass_id' in extra:
        castellan.delete_secret(extra['oozie_pass_id'])
    else:
        LOG.warning("Cluster hasn't Oozie password")


def get_hive_password(cluster):
    cluster = conductor.cluster_get(context.ctx(), cluster)
    extra = cluster.extra.to_dict()
    if 'hive_pass_id' not in extra:
        extra['hive_pass_id'] = u.generate_random_password()
        conductor.cluster_update(context.ctx(), cluster, {'extra': extra})
    return castellan.get_secret(extra['hive_pass_id'])


def delete_hive_password(cluster):
    extra = cluster.extra.to_dict()
    if 'hive_pass_id' in extra:
        castellan.delete_secret(extra['hive_pass_id'])
    else:
        LOG.warning("Cluster hasn't hive password")



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import utils
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import run_scripts as run
from sahara_plugin_vanilla.plugins.vanilla import utils as vu


def start_namenode(cluster):
    nn = vu.get_namenode(cluster)
    _start_namenode(nn)


@utils.event_wrapper(
    True, step=utils.start_process_event_message('NameNode'))
def _start_namenode(nn):
    run.format_namenode(nn)
    run.start_hadoop_process(nn, 'namenode')


def start_secondarynamenode(cluster):
    snn = vu.get_secondarynamenode(cluster)
    if snn:
        _start_secondarynamenode(snn)


@utils.event_wrapper(
    True, step=utils.start_process_event_message("SecondaryNameNodes"))
def _start_secondarynamenode(snn):
    run.start_hadoop_process(snn, 'secondarynamenode')


def start_resourcemanager(cluster):
    rm = vu.get_resourcemanager(cluster)
    if rm:
        _start_resourcemanager(rm)


@utils.event_wrapper(
    True, step=utils.start_process_event_message('ResourceManager'))
def _start_resourcemanager(snn):
    run.start_yarn_process(snn, 'resourcemanager')


def start_historyserver(cluster):
    hs = vu.get_historyserver(cluster)
    if hs:
        run.start_historyserver(hs)


def start_oozie(pctx, cluster):
    oo = vu.get_oozie(cluster)
    if oo:
        run.start_oozie_process(pctx, oo)


def start_hiveserver(pctx, cluster):
    hiveserver = vu.get_hiveserver(cluster)
    if hiveserver:
        run.start_hiveserver_process(pctx, hiveserver)


def start_spark(cluster):
    spark = vu.get_spark_history_server(cluster)
    if spark:
        run.start_spark_history_server(spark)


def start_zookeeper(cluster):
    zk_servers = vu.get_zk_servers(cluster)
    if zk_servers:
        run.start_zk_server(zk_servers)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functools import wraps
from unittest import mock


def mock_event_wrapper(*args, **kwargs):
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            return f(*args, **kwargs)
        return decorated_function
    return decorator


from sahara.plugins import edp
from sahara.plugins import utils as pu
mock.patch('sahara.plugins.utils.event_wrapper', mock_event_wrapper).start()
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import run_scripts as rs
from sahara_plugin_vanilla.tests.unit import base


class RunScriptsTest(base.SaharaTestCase):

    PLUGINS_PATH = 'sahara_plugin_vanilla.plugins.vanilla.hadoop2.'

    def setUp(self):
        super(RunScriptsTest, self).setUp()
        self.instance = mock.Mock()
        self.r = mock.Mock()
        self.remote = mock.Mock(return_value=self.r)
        self.remote.__enter__ = self.remote
        self.remote.__exit__ = mock.Mock()
        self.instance.remote.return_value = self.remote
        pu.event_wrapper = mock_event_wrapper

    @mock.patch(PLUGINS_PATH + 'run_scripts._start_processes')
    @mock.patch('sahara.plugins.context.set_current_instance_id')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    @mock.patch('sahara.plugins.utils.instances_with_services')
    def test_start_dn_nm_processes(self, instances_with_services,
                                   add_provisioning_step,
                                   set_current_instance_id,
                                   _start_processes):
        ins = mock.Mock()
        ins.cluster_id = '111'
        ins.instance_id = '123'
        ins.instance_name = 'ins_1'
        instances = [ins]
        instances_with_services.return_value = instances
        mess = pu.start_process_event_message('DataNodes, NodeManagers')
        ins.node_group.node_processes = ['datanode', 'test']
        rs.start_dn_nm_processes(instances)
        instances_with_services.assert_called_once_with(
            instances, ['datanode', 'nodemanager'])
        add_provisioning_step.assert_called_once_with('111', mess, 1)
        set_current_instance_id.assert_called_once_with('123')
        _start_processes.assert_called_once_with(ins, ['datanode'])

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    def test_start_processes_datanode(self, check_cluster_exists):
        processes = ['datanode']
        rs._start_processes(self.instance, processes)
        self.r.execute_command.assert_called_once_with(
            'sudo su - -c "hadoop-daemon.sh start datanode" hadoop')

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    def test_start_processes_nodemanager(self, check_cluster_exists):
        processes = ['nodemanager']
        rs._start_processes(self.instance, processes)
        self.r.execute_command.assert_called_once_with(
            'sudo su - -c  "yarn-daemon.sh start nodemanager" hadoop')

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    def test_start_processes_both(self, check_cluster_exists):
        processes = ['datanode', 'nodemanager']
        rs._start_processes(self.instance, processes)
        cmd_1 = 'sudo su - -c "hadoop-daemon.sh start datanode" hadoop'
        cmd_2 = 'sudo su - -c  "yarn-daemon.sh start nodemanager" hadoop'
        calls = [mock.call(cmd_1), mock.call(cmd_2)]
        self.r.execute_command.assert_has_calls(calls, any_order=True)

    def test_start_hadoop_process(self):
        process = 'test'
        rs.start_hadoop_process(self.instance, process)
        self.remote.execute_command.assert_called_once_with(
            'sudo su - -c "hadoop-daemon.sh start %s" hadoop' % process)

    def test_start_yarn_process(self):
        process = 'test'
        rs.start_yarn_process(self.instance, process)
        self.remote.execute_command.assert_called_once_with(
            'sudo su - -c  "yarn-daemon.sh start %s" hadoop' % process)

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_start_historyserver(self, add_provisioning_step,
                                 check_cluster_exists):
        rs.start_historyserver(self.instance)
        self.remote.execute_command.assert_called_once_with(
            'sudo su - -c "mr-jobhistory-daemon.sh start historyserver" ' +
            'hadoop')

    @mock.patch(PLUGINS_PATH + 'run_scripts._start_oozie')
    @mock.patch(PLUGINS_PATH + 'run_scripts._oozie_share_lib')
    @mock.patch(PLUGINS_PATH + 'run_scripts._start_mysql')
    @mock.patch(PLUGINS_PATH + 'config_helper.is_mysql_enabled')
    @mock.patch(PLUGINS_PATH + 'utils.get_oozie_password')
    @mock.patch('sahara.plugins.context.set_current_instance_id')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_start_oozie_process(self, add_provisioning_step,
                                 check_cluster_exists,
                                 set_current_instance_id, get_oozie_password,
                                 is_mysql_enabled, _start_mysql,
                                 _oozie_share_lib, _start_oozie):
        self.instance.instance_id = '112233'
        pctx = mock.Mock()
        is_mysql_enabled.return_value = True
        sql_script = pu.get_file_text(
            'plugins/vanilla/hadoop2/resources/create_oozie_db.sql',
            'sahara_plugin_vanilla')
        get_oozie_password.return_value = '123'
        pwd_script = sql_script.replace('password', '123')
        rs.start_oozie_process(pctx, self.instance)
        set_current_instance_id.assert_called_once_with('112233')
        is_mysql_enabled.assert_called_once_with(pctx,
                                                 self.instance.cluster)
        _start_mysql.assert_called_once_with(self.r)
        self.r.write_file_to.assert_called_once_with('create_oozie_db.sql',
                                                     pwd_script)
        self.r.execute_command.assert_called_once_with(
            'mysql -u root < create_oozie_db.sql && '
            'rm create_oozie_db.sql')
        _oozie_share_lib.assert_called_once_with(self.r)
        _start_oozie.assert_called_once_with(self.r)

    @mock.patch(PLUGINS_PATH + 'config_helper.get_spark_home')
    @mock.patch('sahara.plugins.context.set_current_instance_id')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_start_spark_history_server(self, add_provisioning_step,
                                        check_cluster_exists,
                                        set_current_instance_id,
                                        get_spark_home):
        get_spark_home.return_value = '/spark'
        rs.start_spark_history_server(self.instance)
        get_spark_home.assert_called_once_with(self.instance.cluster)
        set_current_instance_id.assert_called_once_with(
            self.instance.instance_id)
        self.r.execute_command.assert_called_once_with(
            'sudo su - -c "bash /spark/sbin/start-history-server.sh" hadoop')

    def test_format_namenode(self):
        rs.format_namenode(self.instance)
        self.remote.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs namenode -format" hadoop')

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_namenode')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_refresh_hadoop_nodes(self, add_provisioning_step,
                                  check_cluster_exists, get_namenode):
        cluster = mock.Mock()
        get_namenode.return_value = self.instance
        rs.refresh_hadoop_nodes(cluster)
        get_namenode.assert_called_once_with(cluster)
        self.remote.execute_command.assert_called_once_with(
            'sudo su - -c "hdfs dfsadmin -refreshNodes" hadoop')

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.'
                'get_resourcemanager')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_refresh_yarn_nodes(self, add_provisioning_step,
                                check_cluster_exists, get_resourcemanager):
        cluster = mock.Mock()
        get_resourcemanager.return_value = self.instance
        rs.refresh_yarn_nodes(cluster)
        get_resourcemanager.assert_called_once_with(cluster)
        self.remote.execute_command.assert_called_once_with(
            'sudo su - -c "yarn rmadmin -refreshNodes" hadoop')

    def test_oozie_share_lib(self):
        cmd_1 = 'sudo su - -c "mkdir /tmp/oozielib && ' \
                'tar zxf /opt/oozie/oozie-sharelib-*.tar.gz -C ' \
                '/tmp/oozielib && ' \
                'hadoop fs -mkdir /user && ' \
                'hadoop fs -mkdir /user/hadoop && ' \
                'hadoop fs -put /tmp/oozielib/share /user/hadoop/ && ' \
                'rm -rf /tmp/oozielib" hadoop'
        cmd_2 = 'sudo su - -c "/opt/oozie/bin/ooziedb.sh ' \
                'create -sqlfile oozie.sql ' \
                '-run Validate DB Connection" hadoop'
        command = [mock.call(cmd_1),
                   mock.call(cmd_2)]
        rs._oozie_share_lib(self.r)
        self.r.execute_command.assert_has_calls(command, any_order=True)

    def test_start_mysql(self):
        rs._start_mysql(self.r)
        self.r.execute_command.assert_called_once_with('/opt/start-mysql.sh')

    def test_start_oozie(self):
        rs._start_oozie(self.r)
        self.r.execute_command.assert_called_once_with(
            'sudo su - -c "/opt/oozie/bin/oozied.sh start" hadoop')

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_namenode')
    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_datanodes')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    @mock.patch('sahara.plugins.utils.plugin_option_poll')
    def test_await_datanodes(self, plugin_option_poll, add_provisioning_step,
                             check_cluster_exists, get_datanodes,
                             get_namenode):
        cluster = mock.Mock()
        get_datanodes.return_value = ['node1']
        r = mock.Mock()
        remote = mock.Mock(return_value=r)
        remote.__enter__ = remote
        remote.__exit__ = mock.Mock()
        namenode = mock.Mock()
        namenode.remote.return_value = remote
        get_namenode.return_value = namenode
        mess = _('Waiting on 1 datanodes to start up')
        test_data = {'remote': r, 'count': 1}
        timeout = config_helper.DATANODES_STARTUP_TIMEOUT
        rs.await_datanodes(cluster)
        get_datanodes.assert_called_once_with(cluster)
        get_namenode.assert_called_once_with(cluster)
        plugin_option_poll.assert_called_once_with(cluster,
                                                   rs._check_datanodes_count,
                                                   timeout, mess, 1, test_data)

    def test_check_datanodes_count(self):
        self.r.execute_command = mock.Mock(return_value=(0, '1'))
        self.assertEqual(rs._check_datanodes_count(self.r, 0), True)

        self.assertEqual(rs._check_datanodes_count(self.r, 1), True)
        self.r.execute_command.assert_called_once_with(
            'sudo su -lc "hdfs dfsadmin -report" hadoop | '
            r'grep \'Live datanodes\|Datanodes available:\' | '
            r'grep -o \'[0-9]\+\' | head -n 1')

    def test_hive_create_warehouse_dir(self):
        rs._hive_create_warehouse_dir(self.r)
        self.r.execute_command.assert_called_once_with(
            "sudo su - -c 'hadoop fs -mkdir -p "
            "/user/hive/warehouse' hadoop")

    def test_hive_copy_shared_conf(self):
        dest = '/root/test.xml'
        rs._hive_copy_shared_conf(self.r, dest)
        self.r.execute_command.assert_called_once_with(
            "sudo su - -c 'hadoop fs -mkdir -p /root && "
            "hadoop fs -put /opt/hive/conf/hive-site.xml "
            "/root/test.xml' hadoop")

    def test_hive_create_db(self):
        rs._hive_create_db(self.r)
        self.r.execute_command.assert_called_once_with(
            'mysql -u root < /tmp/create_hive_db.sql')

    def test_hive_metastore_start(self):
        rs._hive_metastore_start(self.r)
        self.r.execute_command.assert_called_once_with(
            "sudo su - -c 'nohup /opt/hive/bin/hive"
            " --service metastore > /dev/null &' hadoop")

    @mock.patch(PLUGINS_PATH + 'utils.get_hive_password')
    @mock.patch(PLUGINS_PATH + 'config_helper.is_mysql_enabled')
    @mock.patch(PLUGINS_PATH + 'run_scripts._hive_metastore_start')
    @mock.patch(PLUGINS_PATH + 'run_scripts._hive_create_db')
    @mock.patch(PLUGINS_PATH + 'run_scripts._start_mysql')
    @mock.patch(PLUGINS_PATH + 'run_scripts._hive_copy_shared_conf')
    @mock.patch(PLUGINS_PATH + 'run_scripts._hive_create_warehouse_dir')
    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_oozie')
    @mock.patch('sahara.plugins.context.set_current_instance_id')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_start_hiveserver_process(self, add_provisioning_step,
                                      check_cluster_exists,
                                      set_current_instance_id, get_oozie,
                                      _hive_create_warehouse_dir,
                                      _hive_copy_shared_conf, _start_mysql,
                                      _hive_create_db, _hive_metastore_start,
                                      is_mysql_enabled, get_hive_password):
        pctx = mock.Mock()
        path = edp.get_hive_shared_conf_path('hadoop')
        is_mysql_enabled.return_value = True
        cluster = self.instance.cluster
        self.instance.cluster.hadoop_version = '2.7.1'
        ng_cluster = self.instance.node_group.cluster
        get_oozie.return_value = None
        sql_script = pu.get_file_text(
            'plugins/vanilla/v2_7_1/resources/create_hive_db.sql',
            'sahara_plugin_vanilla')
        get_hive_password.return_value = '123'
        pwd_script = sql_script.replace('{{password}}', '123')
        rs.start_hiveserver_process(pctx, self.instance)
        set_current_instance_id.assert_called_once_with(
            self.instance.instance_id)
        _hive_create_warehouse_dir.assert_called_once_with(self.r)
        _hive_copy_shared_conf.assert_called_once_with(self.r, path)
        is_mysql_enabled.assert_called_once_with(pctx, cluster)
        get_oozie.assert_called_once_with(ng_cluster)
        _start_mysql.assert_called_once_with(self.r)
        get_hive_password.assert_called_once_with(cluster)
        self.r.write_file_to.assert_called_once_with(
            '/tmp/create_hive_db.sql', pwd_script)
        _hive_create_db.assert_called_once_with(self.r)
        _hive_metastore_start.assert_called_once_with(self.r)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import testtools

from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import recommendations_utils

CONFIGURATION_SCHEMA = {
    'cluster_configs': {
        'dfs.replication': ('HDFS', 'dfs.replication')
    },
    'node_configs': {
        'yarn.app.mapreduce.am.command-opts': (
            'MapReduce', 'yarn.app.mapreduce.am.command-opts'),
        'yarn.scheduler.maximum-allocation-mb': (
            'YARN', 'yarn.scheduler.maximum-allocation-mb'),
        'yarn.app.mapreduce.am.resource.mb': (
            'MapReduce', 'yarn.app.mapreduce.am.resource.mb'),
        'yarn.scheduler.minimum-allocation-mb': (
            'YARN', 'yarn.scheduler.minimum-allocation-mb'),
        'yarn.nodemanager.vmem-check-enabled': (
            'YARN', 'yarn.nodemanager.vmem-check-enabled'),
        'mapreduce.map.java.opts': (
            'MapReduce', 'mapreduce.map.java.opts'),
        'mapreduce.reduce.memory.mb': (
            'MapReduce', 'mapreduce.reduce.memory.mb'),
        'yarn.nodemanager.resource.memory-mb': (
            'YARN', 'yarn.nodemanager.resource.memory-mb'),
        'mapreduce.reduce.java.opts': (
            'MapReduce', 'mapreduce.reduce.java.opts'),
        'mapreduce.map.memory.mb': (
            'MapReduce', 'mapreduce.map.memory.mb'),
        'mapreduce.task.io.sort.mb': (
            'MapReduce', 'mapreduce.task.io.sort.mb')
    }
}


class TestVersionHandler(testtools.TestCase):
    @mock.patch('sahara.plugins.recommendations_utils.'
                'HadoopAutoConfigsProvider')
    def test_recommend_configs(self, provider):
        f_cluster, f_configs = mock.Mock(), mock.Mock()
        recommendations_utils.recommend_configs(f_cluster, f_configs, False)
        self.assertEqual([
            mock.call(CONFIGURATION_SCHEMA, f_configs, f_cluster, False)
        ], provider.call_args_list)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import six
import testtools

from sahara.plugins import base as pb
from sahara.plugins import exceptions as ex
from sahara.plugins import resource as r
from sahara.plugins import testutils
from sahara_plugin_vanilla.plugins.vanilla.v2_7_5.edp_engine import \
    EdpOozieEngine
from sahara_plugin_vanilla.plugins.vanilla.v2_7_5.edp_engine import \
    EdpSparkEngine
from sahara_plugin_vanilla.plugins.vanilla.v2_7_5 import versionhandler as v_h
from sahara_plugin_vanilla.tests.unit import base


class TestConfig(object):
    def __init__(self, applicable_target, name, default_value):
        self.applicable_target = applicable_target
        self.name = name
        self.default_value = default_value


class VersionHandlerTest(base.SaharaTestCase):
    plugin_path = 'sahara_plugin_vanilla.plugins.vanilla.'
    plugin_hadoop2_path = 'sahara_plugin_vanilla.plugins.vanilla.hadoop2.'

    def setUp(self):
        super(VersionHandlerTest, self).setUp()
        self.cluster = mock.Mock()
        self.vh = v_h.VersionHandler()
        self.override_config("plugins", ["vanilla"])
        pb.setup_plugins()

    def test_get_plugin_configs(self):
        self.vh.pctx['all_confs'] = 'haha'
        conf = self.vh.get_plugin_configs()
        self.assertEqual(conf, 'haha')

    def test_get_node_processes(self):
        processes = self.vh.get_node_processes()
        for k, v in six.iteritems(processes):
            for p in v:
                self.assertIsInstance(p, str)

    @mock.patch(plugin_hadoop2_path +
                'validation.validate_cluster_creating')
    def test_validate(self, validate_create):
        self.vh.pctx = mock.Mock()
        self.vh.validate(self.cluster)
        validate_create.assert_called_once_with(self.vh.pctx,
                                                self.cluster)

    @mock.patch(plugin_path +
                'v2_7_5.versionhandler.VersionHandler.update_infra')
    def test_update_infra(self, update_infra):
        self.vh.update_infra(self.cluster)
        update_infra.assert_called_once_with(self.cluster)

    @mock.patch(plugin_hadoop2_path + 'config.configure_cluster')
    def test_configure_cluster(self, configure_cluster):
        self.vh.pctx = mock.Mock()
        self.vh.configure_cluster(self.cluster)
        configure_cluster.assert_called_once_with(self.vh.pctx, self.cluster)

    @mock.patch(plugin_path + 'v2_7_5.versionhandler.run')
    @mock.patch(plugin_path + 'v2_7_5.versionhandler.starting_scripts')
    @mock.patch('sahara.plugins.swift_helper.install_ssl_certs')
    @mock.patch(plugin_hadoop2_path + 'keypairs.provision_keypairs')
    @mock.patch('sahara.plugins.utils.get_instances')
    @mock.patch('sahara.plugins.utils.cluster_get_instances')
    def test_start_cluster(self, c_get_instances, u_get_instances,
                           provision_keypairs, install_ssl_certs,
                           s_scripts, run):
        self.vh.pctx = mock.Mock()
        instances = mock.Mock()
        c_get_instances.return_value = instances
        u_get_instances.return_value = instances
        self.vh._set_cluster_info = mock.Mock()
        self.vh.start_cluster(self.cluster)
        provision_keypairs.assert_called_once_with(self.cluster)
        s_scripts.start_namenode.assert_called_once_with(self.cluster)
        s_scripts.start_secondarynamenode.assert_called_once_with(self.cluster)
        s_scripts.start_resourcemanager.assert_called_once_with(self.cluster)
        s_scripts.start_historyserver.assert_called_once_with(self.cluster)
        s_scripts.start_oozie.assert_called_once_with(self.vh.pctx,
                                                      self.cluster)
        s_scripts.start_hiveserver.assert_called_once_with(self.vh.pctx,
                                                           self.cluster)
        s_scripts.start_spark.assert_called_once_with(self.cluster)
        run.start_dn_nm_processes.assert_called_once_with(instances)
        run.await_datanodes.assert_called_once_with(self.cluster)
        install_ssl_certs.assert_called_once_with(instances)
        self.vh._set_cluster_info.assert_called_once_with(self.cluster)

    @mock.patch(plugin_hadoop2_path + 'scaling.decommission_nodes')
    def test_decommission_nodes(self, decommission_nodes):
        self.vh.pctx = mock.Mock()
        cluster = mock.Mock()
        instances = mock.Mock()
        self.vh.decommission_nodes(cluster, instances)
        decommission_nodes.assert_called_once_with(self.vh.pctx,
                                                   cluster,
                                                   instances)

    @mock.patch('sahara.plugins.utils.general.get_by_id')
    @mock.patch(plugin_hadoop2_path +
                'validation.validate_additional_ng_scaling')
    @mock.patch(plugin_hadoop2_path +
                'validation.validate_existing_ng_scaling')
    def test_validate_scaling(self, vls, vla, get_by_id):
        self.vh.pctx['all_confs'] = [TestConfig('HDFS', 'dfs.replication', -1)]
        ng1 = testutils.make_ng_dict('ng1', '40', ['namenode'], 1)
        ng2 = testutils.make_ng_dict('ng2', '41', ['datanode'], 2)
        ng3 = testutils.make_ng_dict('ng3', '42', ['datanode'], 3)
        additional = [ng2['id'], ng3['id']]
        existing = {ng2['id']: 1}
        cluster = testutils.create_cluster('test-cluster', 'tenant1', 'fake',
                                           '0.1', [ng1, ng2, ng3])
        self.vh.validate_scaling(cluster, existing, additional)
        vla.assert_called_once_with(cluster, additional)
        vls.assert_called_once_with(self.vh.pctx, cluster, existing)

        ng4 = testutils.make_ng_dict('ng4', '43', ['datanode', 'zookeeper'], 3)
        ng5 = testutils.make_ng_dict('ng5', '44', ['datanode', 'zookeeper'], 1)
        existing = {ng4['id']: 2}
        additional = {ng5['id']}
        cluster = testutils.create_cluster('test-cluster', 'tenant1', 'fake',
                                           '0.1', [ng1, ng4])

        with testtools.ExpectedException(ex.ClusterCannotBeScaled):
            self.vh.validate_scaling(cluster, existing, {})

        get_by_id.return_value = r.create_node_group_resource(ng5)

        with testtools.ExpectedException(ex.ClusterCannotBeScaled):
            self.vh.validate_scaling(cluster, {}, additional)

    @mock.patch(plugin_hadoop2_path + 'scaling.scale_cluster')
    @mock.patch(plugin_hadoop2_path + 'keypairs.provision_keypairs')
    def test_scale_cluster(self, provision_keypairs, scale_cluster):
        self.vh.pctx = mock.Mock()
        instances = mock.Mock()
        self.vh.scale_cluster(self.cluster, instances)
        provision_keypairs.assert_called_once_with(self.cluster,
                                                   instances)
        scale_cluster.assert_called_once_with(self.vh.pctx,
                                              self.cluster,
                                              instances)

    @mock.patch("sahara.plugins.conductor.cluster_update")
    @mock.patch("sahara.plugins.context.ctx")
    @mock.patch(plugin_path + 'utils.get_namenode')
    @mock.patch(plugin_path + 'utils.get_resourcemanager')
    @mock.patch(plugin_path + 'utils.get_historyserver')
    @mock.patch(plugin_path + 'utils.get_oozie')
    @mock.patch(plugin_path + 'utils.get_spark_history_server')
    def test_set_cluster_info(self, get_spark_history_server, get_oozie,
                              get_historyserver, get_resourcemanager,
                              get_namenode, ctx, cluster_update):
        get_spark_history_server.return_value.management_ip = '1.2.3.0'
        get_oozie.return_value.get_ip_or_dns_name = mock.Mock(
            return_value='1.2.3.1')
        get_historyserver.return_value.get_ip_or_dns_name = mock.Mock(
            return_value='1.2.3.2')
        get_resourcemanager.return_value.get_ip_or_dns_name = mock.Mock(
            return_value='1.2.3.3')
        get_namenode.return_value.get_ip_or_dns_name = mock.Mock(
            return_value='1.2.3.4')
        get_namenode.return_value.hostname = mock.Mock(
            return_value='testnode')
        self.vh._set_cluster_info(self.cluster)
        info = {'YARN': {
            'Web UI': 'http://1.2.3.3:8088',
            'ResourceManager': 'http://1.2.3.3:8032'
            },
            'HDFS': {
                'Web UI': 'http://1.2.3.4:50070',
                'NameNode': 'hdfs://testnode:9000'
            },
            'JobFlow': {
                'Oozie': 'http://1.2.3.1:11000'
            },
            'MapReduce JobHistory Server': {
                'Web UI': 'http://1.2.3.2:19888'
            },
            'Apache Spark': {
                'Spark UI': 'http://1.2.3.0:4040',
                'Spark History Server UI': 'http://1.2.3.0:18080'
            }
        }
        cluster_update.assert_called_once_with(ctx(), self.cluster,
                                               {'info': info})

    @mock.patch("sahara.plugins.edp.get_plugin")
    @mock.patch('sahara.plugins.utils.get_instance')
    @mock.patch('os.path.join')
    def test_get_edp_engine(self, join, get_instance, get_plugin):
        job_type = ''
        ret = self.vh.get_edp_engine(self.cluster, job_type)
        self.assertIsNone(ret)

        job_type = 'Java'
        ret = self.vh.get_edp_engine(self.cluster, job_type)
        self.assertIsInstance(ret, EdpOozieEngine)

        job_type = 'Spark'
        ret = self.vh.get_edp_engine(self.cluster, job_type)
        self.assertIsInstance(ret, EdpSparkEngine)

    def test_get_edp_job_types(self):
        job_types = ['Hive', 'Java', 'MapReduce',
                     'MapReduce.Streaming', 'Pig', 'Shell', 'Spark']
        self.assertEqual(self.vh.get_edp_job_types(), job_types)

    def test_get_edp_config_hints(self):
        job_type = 'Java'
        ret = {'job_config': {'args': [], 'configs': []}}
        self.assertEqual(self.vh.get_edp_config_hints(job_type), ret)

    @mock.patch(plugin_hadoop2_path + 'utils.delete_oozie_password')
    @mock.patch(plugin_hadoop2_path + 'keypairs.drop_key')
    def test_on_terminate_cluster(self, delete_oozie_password, drop_key):
        self.vh.on_terminate_cluster(self.cluster)
        delete_oozie_password.assert_called_once_with(self.cluster)
        drop_key.assert_called_once_with(self.cluster)

    @mock.patch(plugin_hadoop2_path + 'config.get_open_ports')
    def test_get_open_ports(self, get_open_ports):
        node_group = mock.Mock()
        self.vh.get_open_ports(node_group)
        get_open_ports.assert_called_once_with(node_group)

    @mock.patch(plugin_hadoop2_path +
                'recommendations_utils.recommend_configs')
    def test_recommend_configs(self, recommend_configs):
        scaling = mock.Mock()
        configs = mock.Mock()
        self.vh.pctx['all_confs'] = configs
        self.vh.recommend_configs(self.cluster, scaling)
        recommend_configs.assert_called_once_with(self.cluster,
                                                  configs,
                                                  scaling)
